###############
##Purpose: Reading in EU emissions tax data for preparing them
## And to then adjust OECD tax data, 
## i.e. splitting them and getting rid of hh tax income
#author: KS, 19.01.2023
###############

#################
##Start reading in WIOD data and setting up
#################
df_eu_env_tax <- read.delim(
  "./data/raw/eurostat/env_ac_taxind2_linear.csv", sep = ",") %>% 
  filter(unit == "MIO_EUR",
         tax == "ENV")

source("./datamanipulation_scripts/manipulate/mappings/sector_mappings.R")
df_country_codes <- read_delim("./data/raw/trade/country_codes_V202201.csv",
                               col_types = c("c","c")) %>% 
  select(iso_3digit_alpha,iso_2digit_alpha) %>% 
  distinct()

### Now build dictionary based on mapping
dic_eu_wiod <- setNames(new_eu_nt,old_eu)

#####
## Add WIOD emissions data to get tax rates in EU
df_ag_NT_E <- readRDS(paste("./data/manipulated/IO/",
                            file_name,"_E.rds",sep="")) %>% 
  select(-c(ends_with("cap_form"),ends_with("invent"))) %>% 
  filter(sup_sector_harm == "CO2") %>% 
  pivot_longer(!c(Country,sup_sector_harm),
               names_to = "country_sector",
               values_to = "Emissions") %>% 
  mutate(use_country = substr(country_sector,1,3),
         use_sector = substr(country_sector,5,13),
         country_sector = NULL) %>% 
  ungroup() %>% 
  select(-c(sup_sector_harm,Country))

#####
###First get the EU tax rates by industry
df_eu_env_tax <- df_eu_env_tax %>% 
  mutate(use_sector = dic_eu_wiod[nace_r2],
         use_sector = if_else(is.na(use_sector),
                                   nace_r2,
                              use_sector))  %>% 
  group_by(geo,use_sector,TIME_PERIOD) %>% 
  summarise(tax_income = sum(OBS_VALUE)) %>% 
  rename(iso_2digit_alpha = geo) %>% 
  left_join(select(df_country_codes,iso_2digit_alpha,iso_3digit_alpha),
            by = c("iso_2digit_alpha")) %>% 
  mutate(iso_3digit_alpha = replace(iso_3digit_alpha,
                                    iso_2digit_alpha == "UK",
                                    "GBR"),
         iso_3digit_alpha = replace(iso_3digit_alpha,
                                    iso_2digit_alpha == "LI",
                                    "LIE"),
         iso_3digit_alpha = replace(iso_3digit_alpha,
                                    iso_2digit_alpha == "EL",
                                    "GRC")) %>% 
  distinct() %>% 
  ungroup() %>% 
  select(-iso_2digit_alpha) %>% 
  rename(use_country = iso_3digit_alpha) 

df_eu_env_tax_ag <- df_eu_env_tax %>% 
  filter(TIME_PERIOD == 2009) %>% 
  left_join(df_ag_NT_E) %>% 
  filter(!is.na(Emissions) ) %>% 
  mutate(tau = tax_income/Emissions) %>% ###In Euros
  filter(!is.na(tau) )

####Now only keep the countries for which I observe all sectors & 
## that have reasonable values
df_all_secs <- df_eu_env_tax_ag %>% 
  group_by(use_country) %>%
  summarize(n_sec = n_distinct(tau)) %>%
  filter(n_sec == max(n_sec)) %>%
  select(-n_sec)

df_count_too_high <- df_eu_env_tax_ag %>% 
  filter(tau>tau_threshold) %>% 
  select(use_country)

#############
## load OECD Xrate data
df_xrate <- read.csv("./data/raw/OECD/x_rates/x_rates.csv",
                       sep = ",",
                       header = TRUE
                       ) %>% 
  tibble() %>% 
  filter(LOCATION == "EA19",  ##Eurozone 
         TIME==2009)

dollar_euro_xrate <- select(df_xrate,Value)[[1]][[1]]

##Now convert ito Dollars and only keep the ones that are not too high
df_eu_env_tax_ag <- df_eu_env_tax_ag %>% 
  filter(use_country %in% df_all_secs$use_country &
           !(use_country %in% df_count_too_high)) %>% 
  mutate(tau = tau*dollar_euro_xrate,
         tax_income = tax_income*dollar_euro_xrate ###Turn tax and tax income into dollar terms
  ) 

##############
###
# Make average EU to use for OECD data below
df_average_eu <- df_eu_env_tax_ag %>% 
  group_by(use_sector) %>% 
  summarise(eu_tau_av = mean(tau,na.rm = T),
            eu_emi = sum(Emissions,na.rm = T),
            eu_tax = sum(tax_income,na.rm = T)) %>% 
  mutate(av_eu_tax = eu_tax/eu_emi) ##Two ways to calculate average: 1. average(tau),
# 2. sum emi and tax income up and get EU wide tau

df_available_eu <- df_eu_env_tax_ag %>% 
  select(use_country,use_sector)
###########################
####Now get the average share that comes from HH consumption
d_average_hh <- df_eu_env_tax %>% 
  filter(use_sector == "TOTAL" | use_sector == "TOTAL_HH_NRES",
         TIME_PERIOD == 2009) %>% 
  pivot_wider(names_from = use_sector,
              values_from = tax_income) %>% 
  mutate(average_hh = TOTAL_HH_NRES/(TOTAL+TOTAL_HH_NRES)) %>% 
  summarise(average_hh = mean(average_hh,na.rm=T)) %>% 
  as.numeric()
########################################
##Add OECD tax revenue
df_oecd_tax_rev <- read.table("./data/raw/OECD/env_tax_rev/ERTR_31012023222956399.csv",
                         sep = ",",
                         header = TRUE) %>% 
  tibble() %>% 
  mutate(Country = NULL) %>% 
  rename(use_country = 1,
         sup_sector_harm = VAR,
         tax_rev = Value) %>% 
  filter(Variable == "Tax revenue, millions USD",
         Year == 2009,
         CAT == "TOT",
         DOM == "CC") %>% 
  select(use_country,tax_rev)

####
# Expand such that it contains all sectors
years <- c(2004:2020)
new_panel <- expand.grid(use_sector = unique(df_eu_env_tax_ag$use_sector),
                         use_country = unique(df_oecd_tax_rev$use_country))

df_oecd_tax_rev_m <- merge(new_panel, df_oecd_tax_rev, all.x = TRUE) %>% 
  left_join(select(df_average_eu,av_eu_tax,use_sector)) %>% 
  left_join(df_ag_NT_E) %>% 
  mutate(tax_rev = (1-d_average_hh) * tax_rev,  #First turn into only production tax revenue
         tau_eu_emi = av_eu_tax *Emissions) %>%  
  anti_join(df_available_eu) ##Only keep the ones that we do not have in the Eurostat data

df_summed_tau_eu_emi <- df_oecd_tax_rev_m %>% 
  group_by(use_country) %>% 
  summarise(sum_tau_eu_emi = sum(tau_eu_emi))

df_oecd_tax_rev_m <- df_oecd_tax_rev_m %>% 
  left_join(df_summed_tau_eu_emi) %>% 
  mutate(interpol_T = tau_eu_emi/sum_tau_eu_emi*tax_rev,  #Calculation based on FL
         interpol_tau = interpol_T/Emissions) %>% 
  filter(!is.na(interpol_tau)) 

#####
# Now combine with EU
df <- df_oecd_tax_rev_m %>% 
  select(use_country,use_sector,interpol_tau) %>% 
  rename(tau = interpol_tau) %>% 
  rbind(select(df_eu_env_tax_ag,use_country,use_sector,tau))


##################################################
write.table(df,
            paste("./data/manipulated/",
                  file_name,
                  "env_tax.csv",
                  sep=""),
            sep=",",
            row.names=FALSE, col.names = TRUE)